import Control.Applicative
import Control.Monad
import System.Environment

data Expr = E | Plus Expr Expr | Times Expr Expr | Minus Expr Expr | Over Expr Expr | ToThe Expr Expr | Log Expr Expr | ThRoot Expr Expr

instance Show Expr where
   showsPrec prec x = let
     (s, pred) = case x of
       E -> (('e':), 11)
       Log x y -> (("log_"++) . showsPrec 11 x . ('(':) . showsPrec 0 y . (')':), 11)
       ToThe x y -> (showsPrec 9 x . ('^':) . showsPrec 8 y, 8)
       ThRoot x y -> (showsPrec 9 x . ('√':) . showsPrec 8 y, 8)
       Times x y -> (showsPrec 6 x . ('*':) . showsPrec 6 y, 6)
       Over x y -> (showsPrec 6 x . ('/':) . showsPrec 7 y, 6)
       Plus x y -> (showsPrec 4 x . ('+':) . showsPrec 4 y, 4)
       Minus x y -> (showsPrec 4 x . ('-':) . showsPrec 5 y, 4)
      in if pred < prec then ('(':) . s . (')':) else s

far :: RealFloat a => a -> a -> Bool
far x y = abs (x - y) > 1e-9 * max (abs x) (abs y)

allAt :: RealFloat a => Int -> [(Expr, a)]
allAt 0 = [(E, exp 1)]
allAt n = do
  (x, a) <- allAt (n-1)
  do
    (y, b) <- allAt (n-1)
    (z, c) <- pure (Plus x y, a + b)
      <|> pure (Times x y, a * b)
      <|> (Minus x y, a - b) <$ guard (far a b)
      <|> (Over x y, a / b) <$ guard (far a b)
      <|> pure (ToThe x y, a ** b)
      <|> (Log x y, logBase a b) <$ guard (far a b)
      <|> pure (ThRoot x y, b ** recip a)
    guard $ not (isNaN c)
    pure (z, c)

allOfThem :: RealFloat a => [(Expr, a)]
allOfThem = [0..] >>= allAt

stalinSortBy :: Ord b => (a -> b) -> [a] -> [a]
stalinSortBy _ [] = []
stalinSortBy f (x:xs) = x : go (f x) xs
  where
    go _ [] = []
    go worst (x:xs)
      | fitness >= worst = go worst xs
      | otherwise = x : go fitness xs
      where fitness = f x

fitness :: RealFloat a => a -> a -> a
fitness target x
  | far target x = abs (target - x)
  | otherwise = 1 / 0

hone :: (Ord a, RealFloat a) => a -> [(Expr, a)]
hone n = stalinSortBy (fitness n . snd) $ allOfThem

main :: IO ()
main = getArgs >>= putStrLn . unlines . map show . hone . read . head
