! Copyright (C) 2025 Aleksander "olus2000" Sabak.
! See https://factorcode.org/license.txt for BSD license.
USING: combinators.short-circuit command-line.parser formatting
grouping.extras io kernel math math.floating-point math.parser
namespaces prettyprint sequences strings ;
IN: fries


<PRIVATE

: 1/ ( n -- 1/n ) 1 swap / ;

PRIVATE>


: >continued ( ratio -- continued )
  dup float? [ double>ratio ] when
  [ { } ] [ 1/ [ dup 0 = not ] [ 1/ 1 /mod swap ] produce nip ]
  if-zero ;


: continued> ( continued -- ratio )
  unclip-last-slice swap <reversed> [ [ 1/ ] dip + ] each ;


: approximations ( continued -- ratios )
  <prefixes> [ continued> ] map ;


: continued>string ( continued -- string )
  unclip-slice number>string swap
  [ "" [ "+1/(%d" sprintf append ] reduce ]
  [ length CHAR: ) <string> ] bi append append ;


: describe-approximation ( continued value -- description )
  [ [ continued>string ] [ continued> ] bi ] dip
  dupd - dup 0 > [ "+" ] [ "-" ] if swap abs
  "x = %s = %u for x = T %s %u" sprintf ;


: describe-approximations ( continued value -- descriptions )
  [ <prefixes> ] dip '[ _ describe-approximation ] map
  "\n" join ;


CONSTANT: full-description-template
"Your target value: T = %u

%s

NOTE: 'exact' match may result from floating-point roundoff error."


: full-description ( continued value -- description )
  tuck describe-approximations
  full-description-template sprintf ;


SYMBOLS: number-of-approximations approximation-target ;


: entry ( -- )
  { T{ option
       { name "--number" }
       { help "Up to how many approximations to print" }
       { type integer }
       { variable number-of-approximations }
       { default 5 }
       { validate [ { [ 0 > ] [ integer? ] } 1&& ] } }
    T{ option
       { name "target" }
       { help "Target number to approximate" }
       { type number }
       { variable approximation-target } } }
  [ approximation-target get dup >continued
    number-of-approximations get index-or-length head
    swap full-description print ] with-options ;


MAIN: entry
